# https://dashscope.console.aliyun.com/model
import json
from typing import Any
from .remote_engine import RemoteEngine
from src import ChatAIEngine as ChatAIEngine

from logging import getLogger
logger = getLogger(__name__)

class ChatAiTongYiQianWen(ChatAIEngine, RemoteEngine):
    API_URL = "https://dashscope.aliyuncs.com/api/v1/services/aigc/text-generation/generation"
    def __init__(self, api_key : str,
                 _model : str = "qwen-1.8b-chat",
                 _is_stream : bool = False,
                 *args, **kwargs) -> None:
        self._modual_name = kwargs.get("model", _model)
        ChatAIEngine.__init__(self, is_stream=kwargs.get("is_stream", _is_stream))
        RemoteEngine.__init__(self, api_key=api_key,
                         base_url=self.API_URL)
    def _transform_raw(self, response):
        return response["output"]["choices"][0]["message"]["content"]

    def _transform_iterator(self, response):
        for chunk in response:
            input_string = chunk.decode('utf-8')
            json_string = input_string[input_string.find("data:") + 5:].strip()
            yield self._transform_raw(json.loads(json_string))

    def process_output(self, data):
        res =  self._transform_iterator(data) if self._is_stream else\
            self._transform_raw(data)
        return super().process_output(res)

    def _execute_raw(self, data: Any) -> Any:
        if self._is_stream:
            self.header['X-DashScope-SSE'] = 'enable'
        response = self.post_prompt({
                'model': self._modual_name,
                "input": {
                    "messages": data
                },
                "parameters": {
                    "result_format": "message",
                    "incremental_output": self._is_stream
                }
            },
            is_stream=self._is_stream)
        if self._is_stream:
            ret = response.iter_content(chunk_size=None)
        else:
            ret = response.json()
        return ret

    def execute(self, data: Any) -> Any:
        return self._execute_raw([{'role': 'user', 'content': f'{data}'}])